@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -23,7 +23,7 @@
     Wrapper types for libudev.  Use ``libudev`` attribute to access libudev
     functions.
 
-    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@googlemail.com>
+    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
 """
 
 
@@ -31,7 +31,7 @@ from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
 import os
-from errno import ENOMEM, EOVERFLOW
+import errno
 from ctypes import (CDLL, Structure, POINTER, get_errno,
                     c_char, c_char_p, c_int, c_ulonglong)
 from ctypes.util import find_library
@@ -84,7 +84,7 @@ dev_t = c_ulonglong
 SIGNATURES = {
     # context
     'udev': dict(
-        new=(None, udev_p),
+        new=([], udev_p),
         unref=([udev_p], None),
         ref=([udev_p], udev_p),
         get_sys_path=([udev_p], c_char_p),
@@ -137,6 +137,7 @@ SIGNATURES = {
         get_sysattr_value=([udev_device_p, c_char_p], c_char_p),
         get_devnum=([udev_device_p], dev_t),
         get_action=([udev_device_p], c_char_p),
+        get_seqnum=([udev_device_p], c_ulonglong),
         get_is_initialized=([udev_device_p], c_int),
         get_usec_since_initialized=([udev_device_p], c_ulonglong),
         get_devlinks_list_entry=([udev_device_p], udev_list_entry_p),
@@ -149,18 +150,23 @@ SIGNATURES = {
         ref=([udev_monitor_p], udev_monitor_p),
         unref=([udev_monitor_p], None),
         new_from_netlink=([udev_p, c_char_p], udev_monitor_p),
-        new_from_socket=([udev_p, c_char_p], udev_monitor_p),
         enable_receiving=([udev_monitor_p], c_int),
-        set_receive_buffer_size=([udev_monitor_p], c_int),
+        set_receive_buffer_size=([udev_monitor_p, c_int], c_int),
         get_fd=([udev_monitor_p], c_int),
         receive_device=([udev_monitor_p], udev_device_p),
         filter_add_match_subsystem_devtype=(
             [udev_monitor_p, c_char_p, c_char_p], c_int),
-        filter_add_match_tag=([udev_monitor_p, c_char_p], c_int))
+        filter_add_match_tag=([udev_monitor_p, c_char_p], c_int),
+        filter_update=([udev_monitor_p], c_int),
+        filter_remove=([udev_monitor_p], c_int))
     }
 
 
-ERRNO_EXCEPTIONS = {ENOMEM: MemoryError, EOVERFLOW: OverflowError}
+ERRNO_EXCEPTIONS = {
+    errno.ENOMEM: MemoryError,
+    errno.EOVERFLOW: OverflowError,
+    errno.EINVAL: ValueError
+}
 
 
 def exception_from_errno(errno):
@@ -245,7 +251,10 @@ ERROR_CHECKERS = dict(
     udev_monitor_enable_receiving=check_errno,
     udev_monitor_receive_device=check_errno_on_null_pointer,
     udev_monitor_filter_add_match_subsystem_devtype=check_negative_errorcode,
-    udev_monitor_filter_add_match_tag=check_negative_errorcode)
+    udev_monitor_filter_add_match_tag=check_negative_errorcode,
+    udev_monitor_filter_update=check_errno,
+    udev_monitor_filter_remove=check_errno,
+)
 
 
 def load_udev_library():
