@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -22,7 +22,7 @@
 
     Device class implementation of :mod:`pyudev`.
 
-    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@googlemail.com>
+    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
 """
 
 from __future__ import (print_function, division, unicode_literals,
@@ -34,7 +34,8 @@ from datetime import timedelta
 
 from pyudev._libudev import libudev
 from pyudev._util import (ensure_byte_string, ensure_unicode_string,
-                          udev_list_iterate, string_to_bool)
+                          udev_list_iterate, string_to_bool,
+                          get_device_type)
 
 
 __all__ = ['Device', 'Attributes', 'Tags',
@@ -45,10 +46,10 @@ __all__ = ['Device', 'Attributes', 'Tags',
 
 class DeviceNotFoundError(LookupError):
     """
-    An error indicating that no :class:`Device` was found.
+    An exception indicating that no :class:`Device` was found.
 
     .. versionchanged:: 0.5
-       Renamed from ``NoSuchDeviceError`` to its current name.
+       Rename from ``NoSuchDeviceError`` to its current name.
     """
 
 
@@ -176,7 +177,8 @@ class Device(Mapping):
         Create a device from a device ``path``.  The ``path`` may or may not
         start with the ``sysfs`` mount point:
 
-        >>> context = pyudev.Context()
+        >>> from pyudev import Context, Device
+        >>> context = Context()
         >>> Device.from_path(context, '/devices/platform')
         Device(u'/sys/devices/platform')
         >>> Device.from_path(context, '/sys/devices/platform')
@@ -199,7 +201,8 @@ class Device(Mapping):
         """
         Create a new device from a given ``sys_path``:
 
-        >>> context = pyudev.Context()
+        >>> from pyudev import Context, Device
+        >>> context = Context()
         >>> Device.from_path(context, '/sys/devices/platform')
         Device(u'/sys/devices/platform')
 
@@ -213,10 +216,10 @@ class Device(Mapping):
 
         .. versionchanged:: 0.4
            Raise :exc:`NoSuchDeviceError` instead of returning ``None``, if
-           no device was found for ``sys_path``
+           no device was found for ``sys_path``.
         .. versionchanged:: 0.5
            Raise :exc:`DeviceNotFoundAtPathError` instead of
-           :exc:`NoSuchDeviceError`
+           :exc:`NoSuchDeviceError`.
         """
         device = libudev.udev_device_new_from_syspath(
             context, ensure_byte_string(sys_path))
@@ -230,11 +233,12 @@ class Device(Mapping):
         Create a new device from a given ``subsystem`` and a given
         ``sys_name``:
 
-        >>> context = pyudev.Context()
-        >>> sda = pyudev.Device.from_name(context, 'block', 'sda')
+        >>> from pyudev import Context, Device
+        >>> context = Context()
+        >>> sda = Device.from_name(context, 'block', 'sda')
         >>> sda
         Device(u'/sys/devices/pci0000:00/0000:00:1f.2/host0/target0:0:0/0:0:0:0/block/sda')
-        >>> sda == pyudev.Device.from_path(context, '/block/sda')
+        >>> sda == Device.from_path(context, '/block/sda')
 
         ``context`` is the :class:`Context` in which to search the device.
         ``subsystem`` and ``sys_name`` are byte or unicode strings, which
@@ -260,6 +264,7 @@ class Device(Mapping):
         ``type``:
 
         >>> import os
+        >>> from pyudev import Context, Device
         >>> ctx = Context()
         >>> major, minor = 8, 0
         >>> device = Device.from_device_number(context, 'block',
@@ -272,7 +277,7 @@ class Device(Mapping):
         Use :func:`os.makedev` to construct a device number from a major and a
         minor device number, as shown in the example above.
 
-        .. note::
+        .. warning::
 
            Device numbers are not unique across different device types.
            Passing a correct number with a wrong type may silently yield a
@@ -300,6 +305,44 @@ class Device(Mapping):
             raise DeviceNotFoundByNumberError(type, number)
         return cls(context, device)
 
+    @classmethod
+    def from_device_file(cls, context, filename):
+        """
+        Create a new device from the given device file:
+
+        >>> from pyudev import Context, Device
+        >>> context = Context()
+        >>> device = Device.from_device_file(context, '/dev/sda')
+        >>> device
+        Device(u'/sys/devices/pci0000:00/0000:00:0d.0/host2/target2:0:0/2:0:0:0/block/sda')
+        >>> device.device_node
+        u'/dev/sda'
+
+        .. warning::
+
+           Though the example seems to suggest that ``device.device_node ==
+           filename`` holds with ``device = Device.from_device_file(context,
+           filename)``, this is only true in a majority of cases.  There *can*
+           be devices, for which this relation is actually false!  Thus, do
+           *not* expect :attr:`~Device.device_node` to be equal to the given
+           ``filename`` for the returned :class:`Device`.  Especially, use
+           :attr:`~Device.device_node` if you need the device file of a
+           :class:`Device` created with this method afterwards.
+
+        ``context`` is the :class:`Context` in which to search the device.
+        ``filename`` is a string containing the path of a device file.
+
+        Return a :class:`Device` representing the given device file.  Raise
+        :exc:`~exceptions.ValueError` if ``filename`` is no device file at all.
+        Raise :exc:`~exceptions.EnvironmentError` if ``filename`` does not
+        exist or if its metadata was inaccessible.
+
+        .. versionadded:: 0.15
+        """
+        device_type = get_device_type(filename)
+        device_number = os.stat(filename).st_rdev
+        return cls.from_device_number(context, device_type, device_number)
+
     @classmethod
     def from_environment(cls, context):
         """
@@ -354,6 +397,15 @@ class Device(Mapping):
         """
         Yield all direct children of this device.
 
+        .. note::
+
+           In udev, parent-child relationships are generally ambiguous, i.e.
+           a parent can have multiple children, *and* a child can have multiple
+           parents. Hence, `child.parent == parent` does generally *not* hold
+           for all `child` objects in `parent.children`. In other words,
+           the :attr:`parent` of a device in this property can be different
+           from this device!
+
         .. note::
 
            As the underlying library does not provide any means to directly
@@ -372,6 +424,21 @@ class Device(Mapping):
             if device != self:
                 yield device
 
+    @property
+    def ancestors(self):
+        """
+        Yield all ancestors of this device from bottom to top.
+
+        Return an iterator yielding a :class:`Device` object for each
+        ancestor of this device from bottom to top.
+
+        .. versionadded:: 0.16
+        """
+        parent = self.parent
+        while parent:
+            yield parent
+            parent = parent.parent
+
     def find_parent(self, subsystem, device_type=None):
         """
         Find the parent device with the given ``subsystem`` and
@@ -407,11 +474,14 @@ class Device(Mapping):
         Return an iterable yielding all parent devices as :class:`Device`
         objects, *not* including the current device.  The last yielded
         :class:`Device` is the top of the device hierarchy.
+
+        .. deprecated:: 0.16
+           Will be removed in 1.0. Use :attr:`ancestors` instead.
         """
-        parent = self.parent
-        while parent:
-            yield parent
-            parent = parent.parent
+        import warnings
+        warnings.warn('Will be removed in 1.0. Use Device.ancestors instead.',
+                      DeprecationWarning)
+        return self.ancestors
 
     @property
     def sys_path(self):
@@ -458,7 +528,9 @@ class Device(Mapping):
            The number is returned as unicode string to preserve the exact
            format of the number, especially any leading zeros:
 
-           >>> device = pyudev.Device.from_path(ctx, '/sys/devices/LNXSYSTM:00')
+           >>> from pyudev import Context, Device
+           >>> context = Context()
+           >>> device = Device.from_path(context, '/sys/devices/LNXSYSTM:00')
            >>> device.sys_number
            u'00'
 
@@ -479,6 +551,7 @@ class Device(Mapping):
         Device type as unicode string, or ``None``, if the device type is
         unknown.
 
+        >>> from pyudev import Context
         >>> context = Context()
         >>> for device in context.list_devices(subsystem='net'):
         ...     '{0} - {1}'.format(device.sys_name, device.device_type or 'ethernet')
@@ -517,6 +590,12 @@ class Device(Mapping):
         this device, and never to any symbolic links to this device node.
         See :attr:`device_links` to get a list of symbolic links to this
         device node.
+
+        .. warning::
+
+           For devices created with :meth:`from_device_file()`, the value of
+           this property is not necessary equal to the ``filename`` given to
+           :meth:`from_device_file()`.
         """
         node = libudev.udev_device_get_devnode(self)
         if node:
@@ -531,6 +610,8 @@ class Device(Mapping):
         Use :func:`os.major` and :func:`os.minor` to decompose the device
         number into its major and minor number:
 
+        >>> import os
+        >>> from pyudev import Context, Device
         >>> context = Context()
         >>> sda = Device.from_name(context, 'block', 'sda')
         >>> sda.device_number
@@ -604,11 +685,59 @@ class Device(Mapping):
         node, or to map a single device into multiple device hierarchies.
         The property provides access to all such symbolic links, which were
         created by UDev for this device.
+
+        .. warning::
+
+           Links are not necessarily resolved by
+           :meth:`Device.from_device_file()`. Hence do *not* rely on
+           ``Device.from_device_file(context, link).device_path ==
+           device.device_path`` from any ``link`` in ``device.device_links``.
         """
         devlinks = libudev.udev_device_get_devlinks_list_entry(self)
         for name, _ in udev_list_iterate(devlinks):
             yield ensure_unicode_string(name)
 
+    @property
+    def action(self):
+        """
+        The device event action as string, or ``None``, if this device was not
+        received from a :class:`Monitor`.
+
+        Usual actions are:
+
+        ``'add'``
+          A device has been added (e.g. a USB device was plugged in)
+        ``'remove'``
+          A device has been removed (e.g. a USB device was unplugged)
+        ``'change'``
+          Something about the device changed (e.g. a device property)
+        ``'online'``
+          The device is online now
+        ``'offline'``
+          The device is offline now
+
+        .. warning::
+
+           Though the actions listed above are the most common, this property
+           *may* return other values, too, so be prepared to handle unknown
+           actions!
+
+        .. versionadded:: 0.16
+        """
+        action = libudev.udev_device_get_action(self)
+        if action:
+            return ensure_unicode_string(action)
+
+    @property
+    def sequence_number(self):
+        """
+        The device event sequence number as integer, or ``0`` if this device
+        has no sequence number, i.e. was not received from a :class:`Monitor`.
+
+        .. versionadded:: 0.16
+        """
+        return libudev.udev_device_get_seqnum(self)
+
     @property
     def attributes(self):
         """
@@ -640,10 +769,13 @@ class Device(Mapping):
         The :class:`Tags` object supports a test for a single tag as well as
         iteration over all tags:
 
+        >>> from pyudev import Context, Device
+        >>> context = Context()
+        >>> device = next(iter(context.list_devices(tag='systemd')))
         >>> 'systemd' in device.tags
         True
         >>> list(device.tags)
-        [u'systemd', u'seat']
+        [u'seat', u'systemd', u'uaccess']
 
         Tags are arbitrary classifiers that can be attached to devices by udev
         scripts and daemons.  For instance, systemd_ uses tags for multi-seat_
