@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -19,19 +19,10 @@
 from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
-from functools import partial
-
 import pytest
 import mock
 
-from pyudev import Enumerator
-
-
-def pytest_generate_tests(metafunc):
-    if 'device' in metafunc.funcargnames:
-        devices = pytest.get_device_sample(metafunc.config)
-        for device_path in devices:
-            metafunc.addcall(id=device_path, param=device_path)
+from pyudev import Enumerator, Device
 
 
 def pytest_funcarg__enumerator(request):
@@ -43,11 +34,13 @@ class TestEnumerator(object):
 
     def test_match_subsystem(self, context):
         devices = context.list_devices().match_subsystem('input')
-        assert all(d.subsystem == 'input' for d in devices)
+        for device in devices:
+            assert device.subsystem == 'input'
 
     def test_match_subsystem_nomatch(self, context):
         devices = context.list_devices().match_subsystem('input', nomatch=True)
-        assert all(d.subsystem != 'input' for d in devices)
+        for device in devices:
+            assert device.subsystem != 'input'
 
     def test_match_subsystem_nomatch_unfulfillable(self, context):
         devices = context.list_devices()
@@ -57,29 +50,34 @@ class TestEnumerator(object):
 
     def test_match_sys_name(self, context):
         devices = context.list_devices().match_sys_name('sda')
-        assert all(d.sys_name == 'sda' for d in devices)
+        for device in devices:
+            assert device.sys_name == 'sda'
 
     def test_match_property_string(self, context):
         devices = list(context.list_devices().match_property('DRIVER', 'usb'))
-        assert all(d['DRIVER'] == 'usb' for d in devices)
-        assert all(d.driver == 'usb' for d in devices)
+        for device in devices:
+            assert device['DRIVER'] == 'usb'
+            assert device.driver == 'usb'
 
     def test_match_property_int(self, context):
         devices = list(context.list_devices().match_property(
             'ID_INPUT_KEY', 1))
-        assert all(d['ID_INPUT_KEY'] == '1' for d in devices)
-        assert all(d.asint('ID_INPUT_KEY') == 1 for d in devices)
+        for device in devices:
+            assert device['ID_INPUT_KEY'] == '1'
+            assert device.asint('ID_INPUT_KEY') == 1
 
     def test_match_property_bool(self, context):
         devices = list(context.list_devices().match_property(
             'ID_INPUT_KEY', True))
-        assert all(d['ID_INPUT_KEY'] == '1' for d in devices)
-        assert all(d.asbool('ID_INPUT_KEY') for d in devices)
+        for device in devices:
+            assert device['ID_INPUT_KEY'] == '1'
+            assert device.asbool('ID_INPUT_KEY')
 
     def test_match_attribute_nomatch(self, context):
         devices = context.list_devices().match_attribute(
             'driver', 'usb', nomatch=True)
-        assert all(d.attributes.get('driver') != 'usb' for d in devices)
+        for device in devices:
+            assert device.attributes.get('driver') != 'usb'
 
     def test_match_attribute_nomatch_unfulfillable(self, context):
         devices = context.list_devices()
@@ -89,7 +87,8 @@ class TestEnumerator(object):
 
     def test_match_attribute_string(self, context):
         devices = list(context.list_devices().match_attribute('driver', 'usb'))
-        assert all(d.attributes['driver'] == b'usb' for d in devices)
+        for device in devices:
+            assert device.attributes['driver'] == b'usb'
 
     def test_match_attribute_int(self, context):
         # busnum gives us the number of a USB bus.  And any decent system
@@ -99,35 +98,37 @@ class TestEnumerator(object):
         # possible.  Still it may fail because the attribute isn't present on
         # any device at all on the system running the test
         devices = list(context.list_devices().match_attribute('busnum', 2))
-        assert all(d.attributes['busnum'] == b'2' for d in devices)
-        assert all(d.attributes.asint('busnum') == 2 for d in devices)
+        for device in devices:
+            assert device.attributes['busnum'] == b'2'
+            assert device.attributes.asint('busnum') == 2
 
     def test_match_attribute_bool(self, context):
         # ro tells us whether a volumne is mounted read-only or not.  And any
         # developers system should have at least one readable volume, thus this
         # test should work on all systems these tests are ever run on
         devices = list(context.list_devices().match_attribute('ro', False))
-        assert all(d.attributes['ro'] == b'0' for d in devices)
-        assert all(not d.attributes.asbool('ro') for d in devices)
+        for device in devices:
+            assert device.attributes['ro'] == b'0'
+            assert not device.attributes.asbool('ro')
 
-    @pytest.need_udev_version('>= 154')
+    @pytest.mark.udev_version('>= 154')
     def test_match_tag_mock(self, context):
-        add_match_tag = 'udev_enumerate_add_match_tag'
         enumerator = context.list_devices()
-        with pytest.patch_libudev(add_match_tag) as add_match_tag:
+        calls = {'udev_enumerate_add_match_tag': [(enumerator, b'spam')]}
+        with pytest.calls_to_libudev(calls):
             retval = enumerator.match_tag('spam')
             assert retval is enumerator
-            add_match_tag.assert_called_with(enumerator, b'spam')
-            args, _ = add_match_tag.call_args
-            assert isinstance(args[1], bytes)
 
-    @pytest.need_udev_version('>= 154')
+    @pytest.mark.udev_version('>= 154')
     def test_match_tag(self, context):
         devices = list(context.list_devices().match_tag('seat'))
-        assert all('seat' in d.tags for d in devices)
+        for device in devices:
+            assert 'seat' in device.tags
 
-    @pytest.need_udev_version('>= 172')
-    def test_match_parent(self, context, device):
+    @pytest.mark.parametrize('device_data', pytest.config.udev_device_sample)
+    @pytest.mark.udev_version('>= 172')
+    def test_match_parent(self, context, device_data):
+        device = Device.from_path(context, device_data.device_path)
         parent = device.parent
         if parent is None:
             pytest.skip('Device {0!r} has no parent'.format(device))
@@ -136,12 +137,12 @@ class TestEnumerator(object):
             assert device in children
             assert parent in children
 
-    @pytest.need_udev_version('>= 165')
-    def test_match_is_initialized(self, context):
-        match_is_initialized = 'udev_enumerate_add_match_is_initialized'
-        with pytest.patch_libudev(match_is_initialized) as match_is_initialized:
-            context.list_devices().match_is_initialized()
-            assert match_is_initialized.called
+    @pytest.mark.udev_version('>= 165')
+    def test_match_is_initialized_mock(self, context):
+        enumerator = context.list_devices()
+        calls = {'udev_enumerate_add_match_is_initialized': [(enumerator,)]}
+        with pytest.calls_to_libudev(calls):
+            enumerator.match_is_initialized()
 
     def test_combined_matches_of_same_type(self, context):
         """
@@ -151,7 +152,9 @@ class TestEnumerator(object):
         devices = context.list_devices()
         for property in properties:
             devices.match_property(property, 'disk')
-        assert all(any(d.get(p) == 'disk' for p in properties) for d in devices)
+        for device in devices:
+            assert (device.get('DEVTYPE') == 'disk' or
+                    device.get('ID_TYPE') == 'disk')
 
     def test_combined_matches_of_different_types(self, context):
         properties = ('DEVTYPE', 'ID_TYPE')
@@ -164,38 +167,39 @@ class TestEnumerator(object):
     def test_match(self, context):
         devices = list(context.list_devices().match(
             subsystem='input', ID_INPUT_MOUSE=True, sys_name='mouse0'))
-        assert all(d.subsystem == 'input' for d in devices)
-        assert all(d.asbool('ID_INPUT_MOUSE') for d in devices)
-        assert all(d.sys_name == 'mouse0' for d in devices)
+        for device in devices:
+            assert device.subsystem == 'input'
+            assert device.asbool('ID_INPUT_MOUSE')
+            assert device.sys_name == 'mouse0'
 
     def test_match_passthrough_subsystem(self, enumerator):
         with mock.patch.object(enumerator, 'match_subsystem',
-                               mocksignature=True) as match_subsystem:
+                               autospec=True) as match_subsystem:
             enumerator.match(subsystem=mock.sentinel.subsystem)
-            match_subsystem.assert_called_with(mock.sentinel.subsystem, False)
+            match_subsystem.assert_called_with(mock.sentinel.subsystem)
 
     def test_match_passthrough_sys_name(self, enumerator):
         with mock.patch.object(enumerator, 'match_sys_name',
-                               mocksignature=True) as match_sys_name:
+                               autospec=True) as match_sys_name:
             enumerator.match(sys_name=mock.sentinel.sys_name)
             match_sys_name.assert_called_with(mock.sentinel.sys_name)
 
     def test_match_passthrough_tag(self, enumerator):
         with mock.patch.object(enumerator, 'match_tag',
-                               mocksignature=True) as match_tag:
+                               autospec=True) as match_tag:
             enumerator.match(tag=mock.sentinel.tag)
             match_tag.assert_called_with(mock.sentinel.tag)
 
-    @pytest.need_udev_version('>= 172')
+    @pytest.mark.udev_version('>= 172')
     def test_match_passthrough_parent(self, enumerator):
         with mock.patch.object(enumerator, 'match_parent',
-                               mocksignature=True) as match_parent:
+                               autospec=True) as match_parent:
             enumerator.match(parent=mock.sentinel.parent)
             match_parent.assert_called_with(mock.sentinel.parent)
 
     def test_match_passthrough_property(self, enumerator):
         with mock.patch.object(enumerator, 'match_property',
-                               mocksignature=True) as match_property:
+                               autospec=True) as match_property:
             enumerator.match(eggs=mock.sentinel.eggs, spam=mock.sentinel.spam)
             assert match_property.call_count == 2
             posargs = [args for args, _ in match_property.call_args_list]
@@ -207,15 +211,16 @@ class TestContext(object):
 
     @pytest.mark.match
     def test_list_devices(self, context):
-        devices = list(context.list_devices(subsystem='input', ID_INPUT_MOUSE=True,
-                                            sys_name='mouse0'))
-        assert all(d.subsystem == 'input' for d in devices)
-        assert all(d.asbool('ID_INPUT_MOUSE') for d in devices)
-        assert all(d.sys_name == 'mouse0' for d in devices)
+        devices = list(context.list_devices(
+            subsystem='input', ID_INPUT_MOUSE=True, sys_name='mouse0'))
+        for device in devices:
+            assert device.subsystem == 'input'
+            assert device.asbool('ID_INPUT_MOUSE')
+            assert device.sys_name == 'mouse0'
 
     @pytest.mark.match
     def test_list_devices_passthrough(self, context):
-        with mock.patch_object(Enumerator, 'match') as match:
+        with mock.patch.object(Enumerator, 'match') as match:
             context.list_devices(subsystem=mock.sentinel.subsystem,
                                  sys_name=mock.sentinel.sys_name,
                                  tag=mock.sentinel.tag,
