@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -19,6 +19,7 @@ from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
 import sys
+import errno
 
 import pytest
 
@@ -100,27 +101,38 @@ def test_udev_list_iterate_no_entry():
 
 
 def test_udev_list_iterate_mock():
-    test_list = ['spam', 'eggs', 'foo', 'bar']
-    get_list_entry = 'udev_enumerate_get_list_entry'
-    with pytest.patch_libudev_list(test_list, get_list_entry) as get_list_entry:
-        items = list(_util.udev_list_iterate(get_list_entry()))
-        assert items == [
-            ('spam', 'value of spam'),
-            ('eggs', 'value of eggs'),
-            ('foo', 'value of foo'),
-            ('bar', 'value of bar')]
+    from pyudev._libudev import libudev
+    items = [('spam', 'eggs'), ('foo', 'bar')]
+    with pytest.libudev_list('udev_enumerate_get_list_entry', items):
+        udev_list = libudev.udev_enumerate_get_list_entry()
+        assert list(_util.udev_list_iterate(udev_list)) == [
+            ('spam', 'eggs'), ('foo', 'bar')]
 
 
 def raise_valueerror():
     raise ValueError('from function')
 
-def test_reraise():
+
+def test_get_device_type_character_device():
+    assert _util.get_device_type('/dev/console') == 'char'
+
+
+def test_get_device_type_block_device():
+    assert _util.get_device_type('/dev/sda') == 'block'
+
+
+def test_get_device_type_no_device_file(tmpdir):
+    filename = tmpdir.join('test')
+    filename.ensure(file=True)
     with pytest.raises(ValueError) as excinfo:
-        try:
-            raise_valueerror()
-        except ValueError as error:
-            assert str(error) == 'from function'
-            tb = sys.exc_info()[2]
-            _util.reraise(ValueError('from except clause'), tb)
-    assert str(excinfo.value) == 'from except clause'
-    assert excinfo.traceback.getcrashentry().name == 'raise_valueerror'
+        _util.get_device_type(str(filename))
+    message = 'not a device file: {0!r}'.format(str(filename))
+    assert str(excinfo.value) == message
+
+
+def test_get_device_type_not_existing(tmpdir):
+    filename = tmpdir.join('test')
+    assert not tmpdir.check(file=True)
+    with pytest.raises(EnvironmentError) as excinfo:
+        _util.get_device_type(str(filename))
+    pytest.assert_env_error(excinfo.value, errno.ENOENT, str(filename))
